Package: qlcMatrix
Version: 0.9.9
Date: 2025-07-23
Title: Utility Sparse Matrix Functions for Quantitative Language
        Comparison
Description: Extension of the functionality of the 'Matrix' package for using sparse matrices. Some of the functions are very general, while other are highly specific for special data format as used for quantitative language comparison.
Authors@R: 
  person(
    "Michael", "Cysouw", 
    email = "cysouw@mac.com", 
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0003-3168-4946")
  )
URL: https://github.com/cysouw/qlcMatrix
BugReports: https://github.com/cysouw/qlcMatrix/issues
License: GPL-3
Encoding: UTF-8
Depends: Matrix (>= 1.2), R (>= 3.2), slam (>= 0.1-32), sparsesvd
Imports: methods, docopt
Suggests: MASS, knitr, rmarkdown
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 6.0.1
NeedsCompilation: no
Packaged: 2025-07-23 12:03:07 UTC; cysouw
Author: Michael Cysouw [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3168-4946>)
Maintainer: Michael Cysouw <cysouw@mac.com>
Repository: CRAN
Date/Publication: 2025-07-23 12:20:09 UTC
Built: R 4.6.0; ; 2026-04-27 11:50:38 UTC; unix
