[![version](http://www.r-pkg.org/badges/version/qlcMatrix)](https://CRAN.R-project.org/package=qlcMatrix)
![downloads](http://cranlogs.r-pkg.org/badges/qlcMatrix)
[![DOI](https://zenodo.org/badge/19068/cysouw/qlcMatrix.svg)](https://zenodo.org/badge/latestdoi/19068/cysouw/qlcMatrix)

qlcMatrix
=========

**Utility sparse matrix functions for Quantitative Language Comparison (QLC)**

This R-package contains various functions that extend the functionality of the Matrix package for using sparse matrices. Some of the functions are very general, while other are highly specific for special data format as used for quantitative language comparison.

The package is available at CRAN, so normally you should just install from within R using:

    install.packages("qlcMatrix")  
    library(qlcMatrix)  

The version here at Github is normally slightly ahead of the CRAN version, and you can install the most up-to-date version using:

    install.packages("devtools")
    devtools::install_github("cysouw/qlcMatrix")

Michael Cysouw
cysouw@mac.com
